
#ifndef _Configurable_8bit_or_16bit_CRC_ASM_dot_h_
#define _Configurable_8bit_or_16bit_CRC_ASM_dot_h_

#include <stdint.h>

/***************************************************************************
   Tke following is code that allows you to configure a cyclic redundancy 
check  based on the the following three parameters.


1) A CRC polynomial. 
2) An initial register state for all of the flip flops
3) Number of bits in CRC polynomial.


This function is written for SPEED not elegance!


 Each of these configuration parameters MUST be changed in the assembly code.
As I type this, the CRC parameters are adjusted to a MODBUS configuration.
 

 There are plenty of notes about CRC listed below!





"Global_input_CRC_data_array_ASM" is an array that has your CRC byte data

"Global_CRC_number_of_bytes" is the number of bytes in your CRC array     



Example:




		Global_input_CRC_data_array_ASM[0]=0X01;//INPUT DATA
		Global_input_CRC_data_array_ASM[1]=0X06;//INPUT DATA

		Global_input_CRC_data_array_ASM[2]=0X01;//INPUT DATA
		Global_input_CRC_data_array_ASM[3]=0X2C;//INPUT DATA

		Global_input_CRC_data_array_ASM[4]=0X04;//INPUT DATA
		Global_input_CRC_data_array_ASM[5]=0XE2;//INPUT DATA

		Global_input_CRC_data_array_ASM[6]=0XCB;// LOW BYTE OF CRC
		Global_input_CRC_data_array_ASM[7]=0X76;// HIGH BYTE CRC


		Global_CRC_number_of_bytes=6;


		Configurable_8_or_16_bit_CRC_ASM() = 0X76CB;



		when 
		Global_CRC_number_of_bytes=8;

		Configurable_8_or_16_bit_CRC_ASM() = 0X000;


****************************************************************************/









uint32_t  Global_input_CRC_data_array_ASM[0XFFFF];

uint32_t  Global_CRC_number_of_bytes;     

uint32_t Configurable_8_or_16_bit_CRC_ASM();





uint32_t Configurable_8_or_16_bit_CRC_ASM()
{  



  uint32_t ASM_generated_CRC;              // Generated product
  uint32_t bit_index_plus_one;             // Loop index
  uint32_t bit_index;                      // derived from a loop index
  uint32_t bit_mask;                       // Used in calculatin reflected crc CONFIG.
  uint32_t reversed_bit_mask;              // Used in calculatin reflected crc CONFIG.
  uint32_t Reflected_CRC_configuration;    // Acutual number that is used in algorythym

  uint32_t byte_array_memory_index;        // Used in byte extraction from array 
  uint32_t number_of_bytes_remaining;      // Used in loop functions
  uint32_t Single_byte_from_CRC_array;     // a single "byte" extracted from user input 





  /*

     //MODBUS configuration See NOTES above
     Number_of_bits_in_CRC   =16;
     Polynomial_CONFIG_WORD  =0X8005;
     Register_Initilisations =0XFFFF;


   */

 uint32_t CRC_REG_CONFIG         =0X8005;
 uint32_t Register_initilisation =0XFFFF;
 uint32_t Number_of_bits_in_CRC  =16;



 

      __asm
   {
     
	   /*

	       First step:
		   Need to generate a "reflected" CRC configuration.
		   For example 0X8005 reflects to 0XA001
	   */


	

	  /******** NUMBER OF BITS IN CRC ***************
	   The initial value of "bit_index" represents
	   the number of bits in the CRC
	  ******** NUMBER OF BITS IN CRC ****************/

	  mov bit_index, 0X10; 

      mov bit_mask,  0X1;           // Incremented 0X0001,0X0002,0X0004,...0X8000
	  mov reversed_bit_mask, 0X8000;// Decremented 0X8000, 0X4000, 0X2000...0X0001
	  mov Reflected_CRC_configuration,0X0000;



      bit_index_loop_start: ;//LOOP BEGIN 
	  {


		 mov eax, CRC_REG_CONFIG;//
		 and eax, bit_mask;
	     cmp eax, 0X0000;

         je  OR_bypass;// 

		      ;// "OR" operation
		      mov ebx, Reflected_CRC_configuration;
		      or ebx, reversed_bit_mask;
			  mov Reflected_CRC_configuration, ebx;

		 OR_bypass:


         shl bit_mask,          0X1;
	     shr reversed_bit_mask, 0X1;


		 ;//******* LOOP CONTROLS ********
	     mov ecx, bit_index;// 
	     dec ecx;
	     mov bit_index, ecx;// 
		 jnz bit_index_loop_start;//
		 ;//******* LOOP CONTROLS ********

	};//END   bit_index_loop_start:
	
	


	  /*

	      The value of "Reflected_CRC_configuration" has been generated

	  */

	    



   



	mov eax, Register_initilisation;//COPY from Register_initilisation
	mov ASM_generated_CRC, eax; //

   
    mov  eax, 0X0000;
    mov byte_array_memory_index, eax;//USED in single byte extraction



    ;//******* LOOP CONTROLS ********
    mov eax, Global_CRC_number_of_bytes;
    mov number_of_bytes_remaining, eax
    ;//******* LOOP CONTROLS ********
    byte_loop_start:
    {

	
	

	      ;// Extract one byte from array
	      mov  ebx,  byte_array_memory_index;
		  mov  eax,  Global_input_CRC_data_array_ASM[ebx];
		  mov Single_byte_from_CRC_array, eax;






		
	  


		 ;// ASM_generated_CRC =  ASM_generated_CRC^ Single_WORD_Extracted_from_input; 
		 ;//ASM_generated_CRC 
		 ;//xor  destination, source
		 mov eax,ASM_generated_CRC;//

		 xor eax,Single_byte_from_CRC_array;
		 mov ASM_generated_CRC,eax;


		  ;//******* LOOP CONTROLS ********
          mov bit_index_plus_one, 0X08;//  
		  ;//******* LOOP CONTROLS ********
	      bit_index_V2_loop_start: ;//LOOP BEGIN 
	      {


		    mov eax,bit_index_plus_one;//
            dec eax;//
		    mov bit_index, eax;//0,1,2,3...7




			


			mov ecx,ASM_generated_CRC;//
			and ecx,0X01;//
            jnz AND_produces_0X01;//
			{
               
			

			   mov eax, ASM_generated_CRC;//
			   shr eax,1;
			   mov ASM_generated_CRC, eax;//
				
			};//
			jmp end_XOR_magic;



            AND_produces_0X01: 
			{
              

			   mov eax, ASM_generated_CRC;//
			   shr eax,1;
			   xor eax, Reflected_CRC_configuration;
			   mov ASM_generated_CRC, eax;//
			   
			}

            end_XOR_magic:;
			











		   ;//******* LOOP CONTROLS ********
	       mov ecx, bit_index_plus_one;// 
	       dec ecx;
	       mov bit_index_plus_one, ecx;// 
		   jnz bit_index_V2_loop_start;//
		   ;//******* LOOP CONTROLS ********

    	};//END   bit_index_V2_loop_start:









		 ;//Increment memory location 
	     mov eax, byte_array_memory_index;
		 add eax, 4;
		 mov byte_array_memory_index, eax;

		 ;//******* LOOP CONTROLS ********
		 dec      number_of_bytes_remaining;
		 mov ecx, number_of_bytes_remaining;// 		 
		 jnz byte_loop_start;//
		 ;//******* LOOP CONTROLS ********

	}
  

	mov eax,ASM_generated_CRC;//

	mov ASM_generated_CRC,eax;





   };//END __asm






   
  
   return ASM_generated_CRC;


}


#endif

/*








 WARNING!!
 Befor reading the following, you must read the external references cited below!!!!





 MODBUS POLYNOMIAL 


    The MODBUS CRC polynomial (Polynomial_CONFIG_WORD)  describes how the
	CRC is calculated.  Find a circuit diagram for a MODBUS CRC.

	Registers 15,2,and 1 have an XOR gate attached to their outputs. The 
	input also has an XOR gate.

    Polynomial_CONFIG_WORD= (X^16)+(X^15)+(X^2)+(X^0); 
  
     The highest value ("16") is is the input XOR gate so you can 
    ignore it during polynomial configuration calculations.

	  Polynomial_CONFIG_WORD = 2^15+ 2^2 + 2^0 = 0X8005;


 MODBUS INITAL STATE


    The initial state of the MODBUS registers is all high.
  
	

Need to change the configuration parameters:



 		
  //MODBUS configuration See NOTES above
   Number_of_bits_in_CRC   =16;
   Polynomial_CONFIG_WORD  =0X8005;
   Register_Initilisations =0XFFFF;



EXAMPLE DATA:

 PAGE 2, of reference #2(References listed below)

"Sent to Write (16-bit) Set Point of 1,250 F" 

Example intilisations 



		Global_input_CRC_data_array_ASM[0]=0X01;//INPUT DATA
		Global_input_CRC_data_array_ASM[1]=0X06;//INPUT DATA

		Global_input_CRC_data_array_ASM[2]=0X01;//INPUT DATA
		Global_input_CRC_data_array_ASM[3]=0X2C;//INPUT DATA

		Global_input_CRC_data_array_ASM[4]=0X04;//INPUT DATA
		Global_input_CRC_data_array_ASM[5]=0XE2;//INPUT DATA

		Global_input_CRC_data_array_ASM[6]=0XCB;// LOW BYTE OF CRC
		Global_input_CRC_data_array_ASM[7]=0X76;// HIGH BYTE CRC


		Global_CRC_number_of_bytes=6;


 Expected_CRC = MODBUS_16bit_CRC(MODBUS_byte_array, Input_number_of_bytes);

 //  Expected_CRC should equal 0X76CB


 MAXIM Semiconductors Application Note 27 Example  



   The " Polynomial_CONFIG_WORD"  describes how the CRC is calculated.
For example, if 

CRC_Polynomial=(X^8) + (X^5) +(X^4)+1; 

  _____________  EQUALS  ___________

CRC_Polynomial=(X^8) + (X^5) +(X^4)+(X^0); 

  
The constant "8" is the highest bit. 

The highest bit is calculated automatically here so you can ignore it.


  "Polynomial_CONFIG_WORD" would be: 
 
  Polynomial_CONFIG_WORD = 2^5+ 2^4 + 2^0;

  Polynomial_CONFIG_WORD = 32 + 16 + 1;

  Polynomial_CONFIG_WORD = 49 ;//(0X31)


  The initial state of the registers are: 0X0000;

  There are 8 bits.
  

Need to change the configuration parameters:


   Number_of_bits_in_CRC   =8;
   Polynomial_CONFIG_WORD  =0X31;
   Register_Initilisations =0X0000;



Example PACKET:


Input_CRC_data_array[0]=0X02;
Input_CRC_data_array[1]=0X1C;
Input_CRC_data_array[2]=0XB8;
Input_CRC_data_array[3]=0X01;
Input_CRC_data_array[4]=0X00;
Input_CRC_data_array[5]=0X00;
Input_CRC_data_array[6]=0X00;



Expected results from above data:CRC =0XA2


















Reference material:


Reference 1:

APPLICATION NOTE 27
Understanding and Using Cyclic Redundancy
Checks with Maxim 1-Wire and iButton Products
Mar 29, 2001

Application Note 27: http://www.maximintegrated.com/an27
APPLICATION NOTE 27, AN27, AN 27, APP27, Appnote27, Appnote 27
 2013 Maxim Integrated Products, Inc.


Reference 2:

"Sample 16-bit Modbus Packet" 


Watlow Winona 2 RGH 02/23/05
1241 Bundy Blvd
Winona, MN 55987
Telephone (507) 494-5656
 2004 Watlow Electric Manufacturing Company



Reference 3:

MODBUS over serial line specification and implementation guide V1.0
Modbus.org http://www.modbus.org/ 1/44
12/02/02






*/