

#include <iostream>
#include <cstdio>
#include <cstring>

/***************  Generate_Gaussian_graph_csv_format *************************

Basic function: Generate a Gaussian plot in Excel from mean and sigma. 

NOTE: csv means Comma Separated Varible

Input values:

Entity Name:            Translation:

"Sigma_value"           Results from experimental data, typically from Excel    
"Mean_average_value"    Results from experimental data, typically from Excel    

"Product_csv_file_name" The name of your product file.



  I programmed the limits to be +4 sigma to -4 sigma. These defaults will represent
>99.99% of the cases that you would normally expect to see. 




  The product values are reported as:

  "Y_axis_plot_values" and "X_axis_plot_values"


  The "Y_axis_plot_values" is what you plot on the "Y" axis.
  The "X_axis_plot_values" is what you plot on the "X" axis.


*************** _________________________________ **************************/



#ifndef _Gaussian_plot_functions_dot_h_
#define _Gaussian_plot_functions_dot_h_






unsigned char Generate_Gaussian_graph_csv_format                ( double       Sigma_value, double Mean_average_value, char Product_csv_file_name[]  );

unsigned char Generate_Gaussian_half_sigma_bin_graph_csv_format(  unsigned int Number_of_Samples, double Sample_data_array[], char Product_csv_file_name[] );

unsigned char Calculate_mean_and_sigma_from_array               ( double       array_of_doubles[], unsigned int number_of_doubles, double &returned_average, double &retured_sigma );






unsigned char Generate_Gaussian_graph_csv_format( double Sigma_value, double Mean_average_value, char Product_csv_file_name[]  )
{

  unsigned char Reply_Value;	 
  unsigned char Return_Message;// 

  double        Max_plot_value;
  double        Min_plot_value;
  double        Step_Value;
  // (Max_plot_value-Min_plot_value)/Number_of_plot_points

  double        Gausian_Y_value;//Exported to *.csv file
  double        Plot_X_value;

  double        Gausian_Prescalar;
  //=1/(Sigama *((2*PI)^0.5))

  double        Gausian_Exponent; 
  //= -((Plot_x_vale-AVERAGE)^2)/(SIGMA^2)

  double        PI_constant = 3.141592653589793;//Close enough.   
  double        Number_of_sigmas_in_plot=4;
  double        Integrated_Area;

  int           Number_of_plot_points;
  int           Data_index_csv_file;//

  FILE          *CSV_FILE_pntr;// Writes the engineering
	                                            // file.


  errno_t       _pvt_File_Open_Error;         // Magical file entity.


	
	
	Number_of_sigmas_in_plot  = 4;  // Default value
	Number_of_plot_points     = 2048;// Default value



 


	  Reply_Value =0X01;
   Return_Message =0X00;// So far so good.

  

   
   




     if   ((0>Sigma_value)||(0==Sigma_value))
		  {
			
			Return_Message =0X01;//Error Message

			fprintf(stderr,"******************** ERROR ALERT ***************************\n");
			fprintf(stderr,"*                                                          *\n");
            fprintf(stderr,"*    ''Generate_Gaussian_graph_csv_format'' puked on you.  *\n");
		    fprintf(stderr,"*                                                          *\n");
			fprintf(stderr,"*  ''Sigma_value'' must be greater than ZERO!              *\n");
			fprintf(stderr,"*                                                          *\n");
			fprintf(stderr,"******************** ERROR ALERT ***************************\n");
			
			return Return_Message;
		  }












      
   _pvt_File_Open_Error = fopen_s(&CSV_FILE_pntr,Product_csv_file_name, "w"); 


		  if ((NULL==CSV_FILE_pntr)||(0!=_pvt_File_Open_Error))
		  {
			
			Return_Message =0X02;//Error Message

			fprintf(stderr,"******************** ERROR ALERT ***************************\n");
			fprintf(stderr,"*                                                          *\n");
            fprintf(stderr,"*    ''Generate_Gaussian_graph_csv_format'' puked on you.  *\n");
		    fprintf(stderr,"*                                                          *\n");
			fprintf(stderr,"*  The file name that you furnished is FUABR               *\n");
			fprintf(stderr,"*                                                          *\n");
			fprintf(stderr,"******************** ERROR ALERT ***************************\n");
			
			return Return_Message;
		  }
		





	

	fprintf(CSV_FILE_pntr,"Y_axis_plot_values,X_axis_plot_values,\n");
	fflush(CSV_FILE_pntr);





	Min_plot_value     =Mean_average_value- Number_of_sigmas_in_plot*Sigma_value;
	Max_plot_value     =Mean_average_value+ Number_of_sigmas_in_plot*Sigma_value;


	Step_Value         =(Max_plot_value-Min_plot_value) /((double)Number_of_plot_points);  




	Gausian_Prescalar = 1.000000/(Sigma_value*(pow((2*PI_constant),0.500000)));

	Integrated_Area =0.0000;

	for (Data_index_csv_file=0;Data_index_csv_file<Number_of_plot_points;Data_index_csv_file++)
	{

		Plot_X_value = Min_plot_value + ((double)Data_index_csv_file)*Step_Value;

	    Gausian_Exponent = -1.00000*(Mean_average_value-Plot_X_value)*(Mean_average_value-Plot_X_value)/(2*Sigma_value*Sigma_value);

		Gausian_Y_value = Gausian_Prescalar * (exp(Gausian_Exponent));


		Integrated_Area = Integrated_Area +(Gausian_Y_value*Step_Value);
	
		fprintf(CSV_FILE_pntr,"%E,",Gausian_Y_value);
	    fflush(CSV_FILE_pntr);


		fprintf(CSV_FILE_pntr,"%E,",Plot_X_value);
	    fflush(CSV_FILE_pntr);


		fprintf(CSV_FILE_pntr,"\n");
	    fflush(CSV_FILE_pntr);
	

	}

	 fclose(CSV_FILE_pntr);

	//fprintf(stderr,"Integrated_Area=%E\n",Integrated_Area);//debug



  return   Reply_Value;
}








unsigned char Calculate_mean_and_sigma_from_array( double array_of_doubles[], unsigned int number_of_doubles, double &returned_average, double &retured_sigma )
{

	unsigned char Reply_value;

	int    Data_index;
	


	double Calculated_average;
	double double_number_of_doubles;
	double SUM_of_squares;
	double ABS_AVE_of_squares;
	double Xi_minus_AVE;
	double Xi_minus_AVE_squared;

	
	Reply_value =0X00;


	if (2>number_of_doubles)
	{
       Reply_value =0X01;

	   fprintf(stderr,"FUBAR alert!: ''number_of_doubles'' must be greater than 1!\n\n\n");
	   fprintf(stderr,"Tap any key and then lament your foolishness!\n");
	   getchar();
	   return  Reply_value;

	}


	double_number_of_doubles = (double) number_of_doubles;


	Calculated_average = 0.0000;

	for (Data_index=0;Data_index<number_of_doubles;Data_index++)

	{
	  Calculated_average = Calculated_average+ ( array_of_doubles[Data_index])/double_number_of_doubles;
	}

	returned_average = Calculated_average;




	SUM_of_squares = 0.00000;
	for (Data_index=0;Data_index<number_of_doubles;Data_index++)

	{

		
		
		 Xi_minus_AVE          = (array_of_doubles[Data_index]-Calculated_average);
		 Xi_minus_AVE_squared  = pow(Xi_minus_AVE,2.000000);
		 SUM_of_squares        = SUM_of_squares + Xi_minus_AVE_squared;



	}

	
	  ABS_AVE_of_squares = SUM_of_squares/(double_number_of_doubles-1.000);



	retured_sigma = pow(ABS_AVE_of_squares,0.50000000);










   return Reply_value;	
}




unsigned char Generate_Gaussian_half_sigma_bin_graph_csv_format(  unsigned int Number_of_Samples, double Sample_data_array[], char Product_csv_file_name[] )
{
     unsigned char  Reply_value;	

	 double STD_DEV;
	 double AVE_MEAS;

	 double bin_zone_array[32];

	 double Upper_limit_of_bin;
     double Lower_limit_of_bin;
	 double Extracted_Sample;

	 double Median_bin_value;

	 unsigned int zone_index;

	 unsigned int Sample_index;

	 unsigned int bin_count_array[32]={0};

	 char Product_csv_file_name_cpy[512];

	   FILE          *BIN_writer_FILE_pntr;// Writes the engineering
	                                            // file.


       errno_t       _pvt_File_Open_Error;         // Magical file entity.


	



	 /*
	    Expect to have data range -4STD_DEV to +4*STD_DEV



	 */




    

      
   _pvt_File_Open_Error = fopen_s(&BIN_writer_FILE_pntr,Product_csv_file_name, "w"); 


		  if ((NULL==BIN_writer_FILE_pntr)||(0!=_pvt_File_Open_Error))
		  {
			
			Reply_value =0X02;//Error Message

			fprintf(stderr,"******************** ERROR ALERT ***************************\n");
			fprintf(stderr,"*                                                          *\n");
            fprintf(stderr,"*    ''Generate_Gaussian_graph_csv_format'' puked on you.  *\n");
		    fprintf(stderr,"*                                                          *\n");
			fprintf(stderr,"*  The file name that you furnished is FUABR               *\n");
			fprintf(stderr,"*                                                          *\n");
			fprintf(stderr,"******************** ERROR ALERT ***************************\n");
			
			return Reply_value;
		  }
		





	 Calculate_mean_and_sigma_from_array( Sample_data_array, Number_of_Samples, AVE_MEAS, STD_DEV );


	 for (zone_index=0;zone_index<64;zone_index++)
	 {
		 bin_zone_array [zone_index]= (AVE_MEAS-4.00*STD_DEV) +STD_DEV*((double)zone_index);

		 //fprintf(stderr,"bin_zone_array[%i]=%E\n",zone_index, bin_zone_array [zone_index]);

	 }


	 
	 

	  for (zone_index=0;zone_index<15;zone_index++)
	 {
		

		 Lower_limit_of_bin = bin_zone_array [zone_index];
		 Upper_limit_of_bin = bin_zone_array [(zone_index+1)];


		 bin_count_array[zone_index]=0;

		 for (Sample_index=0;Sample_index<Number_of_Samples;Sample_index++)
		 {

			 Extracted_Sample = Sample_data_array[Sample_index];


			 if (Lower_limit_of_bin<Extracted_Sample)
			 {

				 if (Upper_limit_of_bin>Extracted_Sample)
				 {
                      bin_count_array[zone_index] = bin_count_array[zone_index] +1;
 
				 }

			 }

           
		 }



		// fprintf(stderr,"bin_count_array[%i]=%i\n",zone_index,  bin_count_array[zone_index]);


	 };//END  for (zone_index=0;zone_index<15;zone_index++)





	  	

	    fprintf(BIN_writer_FILE_pntr,"X_axis_value_at_bin,Y_axis_bin_count,\n");
	    fflush(BIN_writer_FILE_pntr);


	    for (zone_index=0;zone_index<15;zone_index++)
     	 {


          Lower_limit_of_bin = bin_zone_array [zone_index];
		  Upper_limit_of_bin = bin_zone_array [(zone_index+1)];

		  Median_bin_value =(Upper_limit_of_bin + Lower_limit_of_bin)/2.00000;

		  fprintf(BIN_writer_FILE_pntr,"%E,",Median_bin_value);
		  fprintf(BIN_writer_FILE_pntr,"%i,",  bin_count_array[zone_index] );
		  fprintf(BIN_writer_FILE_pntr,"\n",Median_bin_value);

		   //bin_count_array[zone_index]
		
		 }

		fclose(BIN_writer_FILE_pntr);


		

		strcpy(Product_csv_file_name_cpy,"HISTO_GRAPH_plot_");

		strcat(Product_csv_file_name_cpy,Product_csv_file_name);

		Generate_Gaussian_graph_csv_format ( STD_DEV, AVE_MEAS, Product_csv_file_name_cpy );

	

	 Reply_value =0X00;

	  return Reply_value;	
}



#endif