
/**************** _MS_VS_2005_RS232_manager_dot_h_ *******************

  Windows based RS232 (aka COM) control.  Designed for use with data 
aquisition. Some of the more obscure or derelict data formats are not
implemented.

Some of the protocols not implemented are :

1) RTS and CTS 
2) Wait for termination character.eg \r or \n
3) XON / XOFF



Example usage:



   MS_VS_2005_RS232_manager  FLUKE_1504_RS232_class;//Adjusts RS232 configuration I generated this POC



   FLUKE_1504_RS232_class.COM_port_number(13);
   FLUKE_1504_RS232_class.BAUD_ADJUST(9600);
   FLUKE_1504_RS232_class.data_bits_8();
   FLUKE_1504_RS232_class.Parity_NONE();
   FLUKE_1504_RS232_class.STOP_bits_ONE();
   FLUKE_1504_RS232_class.Hardware_latency_mS(1234);
   FLUKE_1504_RS232_class.Generate_RS232_conduit();

   *
   *
   *

   FLUKE_1504_RS232_class.Transmitt_char_array_to_RS232(RS232_message,Number_of_Bytes);
   *
   *
   FLUKE_1504_RS232_class.Receive_char_array_from_RS232(RS232_message,Number_of_Bytes);


*/




/*********************  Silence the Complier Snivel Function ******************************************

   I used an older form of "strcpy_s".  Specifically, I used "strcpy"  the compiler will snivel about
   the safety of this function"warning C4996: 'strcpy': This function or variable may be unsafe."
   The compiler directives below silences some of  the compiler's snivel functions.

*********************  ____________________________________ *******************************************/



#ifndef _CRT_SECURE_NO_WARNINGS 
#define _CRT_SECURE_NO_WARNINGS
#pragma message("\n ************* ------------------------- ***********************  \n")
#pragma message("\n **    COMPILER WARNING FUNCTION DEACTIVATED                   *  \n")
#pragma message("\n **              January, 2, 2014                              *  \n")
#pragma message("\n ************* _________________________ ***********************  \n \n \n")
#endif



#include <windows.h>
#include <stdio.h>
#include <winbase.h>
#include <WinNT.h>
#include <WinDef.h>
#include <stdlib.h>
#include <tchar.h>
#include <strsafe.h>


#ifndef   define_256_chars  
#define   define_256_chars  (256)
#endif




#ifndef _MS_VS_2005_RS232_manager_dot_h_
#define _MS_VS_2005_RS232_manager_dot_h_

class MS_VS_2005_RS232_manager
{
   
	 
 public:
     
      MS_VS_2005_RS232_manager();
	  ~MS_VS_2005_RS232_manager();

	  void          Report_availible_COM_ports();//

	  void          COM_port_number(int COM_port_number);

	  void          BAUD_ADJUST    (unsigned int BAUD);

    
	  void          data_bits_5();
	  void          data_bits_6();
	  void          data_bits_7();
	  void          data_bits_8();


	  void          Parity_EVEN();
	  void          Parity_ODD();
	  void          Parity_NONE();

 
	  void          STOP_bits_ONE();
	  void          STOP_bits_TWO();

	  void          Hardware_latency_mS(DWORD TIME_out_miliseconds);

	  unsigned char Generate_RS232_conduit();



	 unsigned char  Transmitt_char_array_to_RS232( char char_array[], int &Desired_number_of_chars);
	 unsigned char  Receive_char_array_from_RS232( char char_array[], int &Desired_number_of_chars );


	 void           Obliterate_RS232_conduit();


	 void           Report_existing_COM_port_adjustments(void);



 private:
	 unsigned char  _pvt_PORT_adjustments_monitor;// 
  
	 /* NOTES on  "_pvt_PORT_adjustments_monitor"  

	   bit value:   Translation:

	   0X01         PORT has been selected 
	   0X02         BAUD has been selected
	   0X04         Number of data bits has been selected.
	   0X08         Parity has been selected
	   0X10         Stop bits have been selected
	   0X20         TIMEOUTS selected

	 */




	 char            _pvt_convert_int_to_char( int int_ZERO_to_NINE);  
	 char            _pvt_COM_port_full_ID_array[define_256_chars];  // Example "\\\\.\\COM16"
	 char            _pvt_COM_NUM_ID_array[define_256_chars];  // Example "\\\\.\\COM16"
	 char            _pvt_BAUD_array[define_256_chars];
	 char            _pvt_data_bits_array[define_256_chars];
	 char            _pvt_Parity_array[define_256_chars];
	 char            _pvt_Stop_bits_array[define_256_chars];
	 char            _pvt_DTR_RTS_circuit_CONFIG_array[define_256_chars];

	 HANDLE          _pvt_COM_port_file_pntr;// "HANDLE" inside of WinNT.h

	 COMMTIMEOUTS    _pvt_COM_port_latency_magic; // "COMMTIMEOUTS" inside of the WinBase.h header


	 unsigned char    _pvt_RS232_conduit_has_been_generated;
		

	 unsigned char    _pvt_User_selections_Examination();// TESTS RS232 adjustments


	  DWORD           _pvt_Time_out_miliseconds;


	 unsigned char    _pvt_char_array_to_unsigned_char_array(char Source_char_array[], unsigned char Destination_unsigned_char_array[], unsigned int Array_Length);
     unsigned char    _pvt_unsigned_char_array_to_char_array(unsigned char Source_unsigned_char_array[], char Destination_char_array[], unsigned int Array_Length);



};//END  MS_VS_2005_RS232_manager




















   MS_VS_2005_RS232_manager::MS_VS_2005_RS232_manager()
  {
	 _pvt_PORT_adjustments_monitor = 0X00;// Nothing is selected yet
    
	
	 _pvt_RS232_conduit_has_been_generated =0X00;//FALSE

	 _pvt_Time_out_miliseconds = 1000;
    
  };//END MS_VS_2005_RS232_manager()









   MS_VS_2005_RS232_manager::~MS_VS_2005_RS232_manager()
  {

     system("color 0C");

	 printf( "Thats all folks!!!");

	 if ((INVALID_HANDLE_VALUE!=_pvt_COM_port_file_pntr)&&(0!=_pvt_RS232_conduit_has_been_generated))
	 {
	    CloseHandle(_pvt_COM_port_file_pntr);// NEED to fix this.
	 }


   };//END ~MS_VS_2005_RS232_manager()





   unsigned char MS_VS_2005_RS232_manager::Generate_RS232_conduit()
   {
     
     unsigned char Reply_message;
	 char          COM_PORT_error_message[define_256_chars];
	 char          COM_PORT_CONFIG_array[4096];
   

	 unsigned char Prerequisite_Flag; 



	 Reply_message =0X00;


	 // Close any open file pointers.
	 if (INVALID_HANDLE_VALUE==_pvt_COM_port_file_pntr)
	 {
	    CloseHandle(_pvt_COM_port_file_pntr);
	 }


	
	 	

	
	 // Test for RS232 settings
	 Prerequisite_Flag = _pvt_User_selections_Examination();//
	  
	 
	 if (0X00!=Prerequisite_Flag)
	 {
		 return 0X01;
	 }


	 	
	 /*****************************  DO NOT DELETE **************************************************

	 "MODE" Notes:
	 Expected format:
	 COMx[:][baud=b][parity=p][data=d][stop=s][to={on|off}][xon={on|off}][odsr={on|off}][octs={on|off}][dtr={on|off|hs}][rts={on|off|hs|tg}][idsr={on|off}]

	 Example:
	 mode com13 baud=115200 parity=n data=8 stop=1 to=off xon=off odsr=off octs=off dtr=on rts=on idsr=off 

	
	 *****************************  _____________ ***************************************************/




	 // Generate configuration string. 
	 strncpy(COM_PORT_CONFIG_array,"mode",256);                //MODE
	 strncat(COM_PORT_CONFIG_array," ",256);


	 strncat(COM_PORT_CONFIG_array,_pvt_COM_NUM_ID_array,256); //COM 
	 strncat(COM_PORT_CONFIG_array," ",256);

	 strncat(COM_PORT_CONFIG_array,_pvt_BAUD_array,256);       //BAUD       
	 strncat(COM_PORT_CONFIG_array," ",256);

	 strncat(COM_PORT_CONFIG_array,_pvt_Parity_array,256);     //PARITY
	 strncat(COM_PORT_CONFIG_array," ",256);

	 strncat(COM_PORT_CONFIG_array,_pvt_data_bits_array,256);  //DATA BITS
	 strncat(COM_PORT_CONFIG_array," ",256);
	  
	 
	 strncat(COM_PORT_CONFIG_array,_pvt_Stop_bits_array,256);  //STOP BITS
	 strncat(COM_PORT_CONFIG_array," ",256);

	 	
	  


	 strncat(COM_PORT_CONFIG_array,"to=off xon=off odsr=off octs=off dtr=on rts=on idsr=off ",256);  //remainder are defaults
	 strncat(COM_PORT_CONFIG_array," ",256);





	  system(COM_PORT_CONFIG_array);





	  
         // Microsoft magic follows .... Ooohh GAH Boooo GAH Shazaam!!!
        _pvt_COM_port_file_pntr = CreateFileA(_pvt_COM_port_full_ID_array,
                                              GENERIC_READ|GENERIC_WRITE,
                                              0,                          /* no share  */
                                              NULL,                       /* no security */
                                              OPEN_EXISTING,
                                              0,                          /* no threads */
                                              NULL);                      /* no templates */



      if(_pvt_COM_port_file_pntr==INVALID_HANDLE_VALUE)
	  {

		  strncpy(COM_PORT_error_message,"''MS_VS_2005_RS232_manager.COM_port_number(",256);
		  strncat(COM_PORT_error_message,_pvt_COM_NUM_ID_array,256);
		  strncat(COM_PORT_error_message,")",256);

		   printf("\n\n");

		  printf("****** ERROR ALERT  ERROR ALERT  ERROR ALERT  ***********\n");
		  printf("*                                                       *\n");
		  printf("* %s\n",COM_PORT_error_message);
		  printf("*           PUKED on your request!!!                    *\n");
		  printf("****** ERROR ALERT  ERROR ALERT  ERROR ALERT  ***********\n");

		  printf("\n\n\n\n\n");

		 
		  
		  printf("TAP any key to continue.......");
		  getchar();


		   return 0X02;

	  };// END Error message


	 

	  

       _pvt_COM_port_latency_magic.ReadIntervalTimeout         = _pvt_Time_out_miliseconds;
       _pvt_COM_port_latency_magic.ReadTotalTimeoutMultiplier  = 0;
       _pvt_COM_port_latency_magic.ReadTotalTimeoutConstant    = 0;
       _pvt_COM_port_latency_magic.WriteTotalTimeoutMultiplier = 0;
       _pvt_COM_port_latency_magic.WriteTotalTimeoutConstant   = 0;

       if(!SetCommTimeouts(_pvt_COM_port_file_pntr, &_pvt_COM_port_latency_magic))
       {
         printf("ERROR: Unable to adjust latency values!\n");
         CloseHandle(_pvt_COM_port_file_pntr);
         return 0X04;
        }



        _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor | 0X20;//

	   _pvt_RS232_conduit_has_been_generated =0XFF;//TRUE


	 return Reply_message;
   }

   char MS_VS_2005_RS232_manager::_pvt_convert_int_to_char( int int_ZERO_to_NINE)
   {
	  char Return_char;

     if (9<int_ZERO_to_NINE)
	 {
          printf("* ''_pvt_convert_int_to_char'' only accepts values ZERO to NINE!!! *\n");

		  Return_char ='Z';// Bogus value

		  return Return_char;
	 }


	  if (0>int_ZERO_to_NINE)
	 {
          printf("* ''_pvt_convert_int_to_char'' only accepts values ZERO to NINE!!! *\n");

		  Return_char ='Z';// Bogus value

		  return Return_char;
	 }

	  Return_char =(char) (int_ZERO_to_NINE+48);//See ASCII table

	  return  Return_char;

   }


   void MS_VS_2005_RS232_manager::Report_availible_COM_ports()
   {
    
	  FILE *RS232_com_report_FILE_pntr;
	 
	  errno_t    File_Open_Error;             // Magical varible: Abra Cadabra

	  File_Open_Error = fopen_s(&RS232_com_report_FILE_pntr,"RS232_avalibility_report.txt", "w+"); 

	  if (0!=File_Open_Error)
	  {
          printf("****** ERROR ALERT  ERROR ALERT  ERROR ALERT  ***********\n");
		  printf("******                                        ***********\n");
		  printf("******  Program can not open file named:      ***********\n");
		  printf("******                                        ***********\n");
		  printf("******  ''RS232_avalibility_report.txt''      ***********\n");
		  printf("******                                        ***********\n");
		  printf("****** Is there another instance of this file ***********\n");
		  printf("****** open??                                 ***********\n");
		  printf("\n\n\n\n\n");
		  printf("TAP any key to continue.......");
		  getchar();

        return;//

	  }

     fclose(RS232_com_report_FILE_pntr);

	 printf("WARNING: this is a VERY slow function.\n");
	 printf("Expect up to 30 seconds of delay.\n\n\n");
	 
	 printf("Also you can search in Control panel\\device manager.\n");
	 printf("These controls are typically in the  lower left hand \n");
	 printf("corner of your windows enviroment.\n\n");
	  printf("Good luck with finding this window! \n\n\n\n");


	 system("mode > RS232_avalibility_report.txt ");// SEE NOTES (above)

     system("RS232_avalibility_report.txt ");




     return;
   };// END Report_availible_COM_ports()




   void          MS_VS_2005_RS232_manager::COM_port_number(int COM_port_number)
   {
	   int TENs_place;// used in reformatiing 
	   int ONEs_place;// used in reformatiing 
	   char Number_array[define_256_chars];

	  
	   char TENs_char;
	   char ONEs_char;


      _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor & (0XFE);
	  // Purge COM PORT OK bit 

     
	   if ((0>COM_port_number)||(30<COM_port_number))
	   {

          printf("****** ERROR ALERT  ERROR ALERT  ERROR ALERT  ***********\n");
		  printf("**                                                     **\n");
		  printf("**  ''COM_port_number'' Only accepts values    **\n");
		  printf("**  between 0(ZERO) and 30(THIRTY)                     **\n");
		  printf("**                                                     **\n");
		  printf("****** ERROR ALERT  ERROR ALERT  ERROR ALERT  ***********\n");

		  printf("\n\n\n\n\n");
		  printf("TAP any key to continue.......");
		  getchar();

		   return;
	   }
   

       _pvt_COM_port_full_ID_array[256];  // Example "\\\\.\\COM16" define_256_chars


	   strncpy( _pvt_COM_port_full_ID_array,"\\\\.\\COM",define_256_chars);


	   TENs_place = COM_port_number/10;
	   ONEs_place = COM_port_number - 10*TENs_place;


	   TENs_char = _pvt_convert_int_to_char( TENs_place);
	   ONEs_char = _pvt_convert_int_to_char( ONEs_place);
     

	   

	   if (0==TENs_place)
	   {

		   Number_array[0]=ONEs_char;
		   Number_array[1]='\0';
	   }else
	   {

		   Number_array[0]=TENs_char;
		   Number_array[1]=ONEs_char;
		   Number_array[2]='\0';

	   }

	    strncat( _pvt_COM_port_full_ID_array,Number_array,10);
		
		strncpy(_pvt_COM_NUM_ID_array,"COM",256);
		strncat(_pvt_COM_NUM_ID_array,Number_array,256);

		  


	  _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor |0X01;
	  // COM port selection OK

	

	   return ;
   };//END MS_VS_2005_RS232_manager::COM_port_number(int COM_port_number)





   




    void   MS_VS_2005_RS232_manager::data_bits_5()
    {

		strncpy(_pvt_data_bits_array, " data=5",50);
		_pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X04;

    }


	void   MS_VS_2005_RS232_manager::data_bits_6()
    {

		strncpy(_pvt_data_bits_array, " data=6",50);
		_pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X04;
    }

	void   MS_VS_2005_RS232_manager::data_bits_7()
    {

		strncpy(_pvt_data_bits_array, " data=7",50);
		_pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X04;
    }

	void   MS_VS_2005_RS232_manager::data_bits_8()
    {

		strncpy(_pvt_data_bits_array, " data=8",50);
		_pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X04;
    }




	void   MS_VS_2005_RS232_manager::Parity_EVEN()
	{

        strncpy(_pvt_Parity_array, "parity=e",50);
       _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X08;

	  
	}

	void   MS_VS_2005_RS232_manager::Parity_ODD()
	{

        strncpy(_pvt_Parity_array, "parity=o",50);
       _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X08;       
	}

	void   MS_VS_2005_RS232_manager::Parity_NONE()
	{

        strncpy(_pvt_Parity_array, "parity=n",50);
       _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X08;       
	}
	


 
    void    MS_VS_2005_RS232_manager::STOP_bits_ONE()
	{
	
          strncpy(_pvt_Stop_bits_array, " stop=1",50);
		 _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X10; 

		 

	}

    void    MS_VS_2005_RS232_manager::STOP_bits_TWO()
	{
	
          strncpy(_pvt_Stop_bits_array, " stop=2",50);
		 _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X10; 

	}


	 void MS_VS_2005_RS232_manager::BAUD_ADJUST(unsigned int BAUD)
	 {

	     char BAUD_char_array_form[256];


		 sprintf_s(BAUD_char_array_form,50,"%d",BAUD);
		 strncpy(_pvt_BAUD_array," baud=",256);
		 strncat(_pvt_BAUD_array,BAUD_char_array_form,256);

		 _pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor|0X02;
        
	 }


	  unsigned char    MS_VS_2005_RS232_manager::_pvt_User_selections_Examination()
	  {

		  unsigned char Reply_message;

	     /* NOTES on  "_pvt_PORT_adjustments_monitor"  

	      bit value:   Translation:

	      0X01         PORT has been selected 
	      0X02         BAUD has been selected
	      0X04         Number of data bits has been selected.
	      0X08         Parity has been selected
	      0X10         Stop bits have been selected
		  0X20         TIMEOUTS NOT selected

	     */

		Reply_message = 0X00;

		if (0==(0X01&_pvt_PORT_adjustments_monitor))
		{
			printf("\n\n NO COM PORT HAS BEEN SELECTED.\n");
			Reply_message = Reply_message | 0X01;
		}

	    
		if (0==(0X02&_pvt_PORT_adjustments_monitor))
		{
			printf("\n\n BAUD RATE HAS NOT BEEN SELECTED.\n");
			Reply_message = Reply_message | 0X02;
		}


		
		if (0==(0X04&_pvt_PORT_adjustments_monitor))
		{
			printf("\n\n NUMBER of DATA BITS HAS NOT BEEN SELECTED.\n");
			Reply_message = Reply_message | 0X04;
		}

	    
		if (0==(0X08&_pvt_PORT_adjustments_monitor))
		{
			printf("\n\n PARITY HAS NOT BEEN SELECTED.\n");
			Reply_message = Reply_message | 0X08;
		}

		if (0==(0X10&_pvt_PORT_adjustments_monitor))
		{
			printf("\n\n STOP BITS HAVE NOT BEEN SELECTED.\n");
			Reply_message = Reply_message | 0X10;
		}


		if (0==(0X20&_pvt_PORT_adjustments_monitor))
		{
			printf("\n\n TIMEOUT (latency) has not been selected.\n");
			Reply_message = Reply_message | 0X20;
		}

	

		  return Reply_message;
	  }




	  /******************   Transmitt_char_array_to_RS232  *********************************

	  Entity Name:               Entity Type:  Translation to English:

	  "char_array"               Input          An array of chars that you want delivered
	                                           to the RS232 conduit.

     "Desired_number_of_chars"   Input         The number of chars that you want delivered 
	                                          to the RS232 conduit.    
    
	  "Desired_number_of_chars"  Output        The number of chars that were actually 
	                                          deliverd to the RS232 conduit.  

  
	    Function return value:      Translation:

	  !=0X00                     Not all of the RS232 adjustments were made
	   0X01                      PORT has been selected 
	   0X02                      BAUD has been selected
	   0X04                      Number of data bits has been selected.
	   0X08                      Parity has been selected
	   0X10                      Stop bits have been selected
	   0X20                      TIMEOUTS NOT selected
	   0X30                      Write fucntion reported an error



	  ******************  _____________________________  **********************************/

	   unsigned char MS_VS_2005_RS232_manager::Transmitt_char_array_to_RS232( char char_array[], int &Desired_number_of_chars)
	   {
		   unsigned char Reply_Value;

		   unsigned char Ausgang_buffer[0XFFFF];// never expect to transmitt more than 0XFFFF!!

		   int          Actual_number_of_bytes_written;

		   int         Write_Sucess_Flag;
		   Reply_Value=0X00;



		    Reply_Value =_pvt_User_selections_Examination();// DO NOT proceed if COM PORT does not work

			if (0X00!= Reply_Value)
			{
				return Reply_Value;
			}

			

			if (0X00==_pvt_RS232_conduit_has_been_generated)
			{

				Generate_RS232_conduit();//Implicit desire for a functioning conduit

			}




			_pvt_char_array_to_unsigned_char_array(char_array, Ausgang_buffer, (unsigned int) Desired_number_of_chars );

             Write_Sucess_Flag = WriteFile(_pvt_COM_port_file_pntr, Ausgang_buffer, Desired_number_of_chars, (LPDWORD)((void *)&Actual_number_of_bytes_written), NULL);

		

		 
			if (0==Write_Sucess_Flag)
			{
				printf("MS_VS_2005_RS232_manager::Transmitt_char_array_to_RS232 has reported a problem!\n");

				Reply_Value = 0X80;//
				
			}


			Desired_number_of_chars = Actual_number_of_bytes_written;


		   return Reply_Value;
	   }




	   /******************   Transmitt_char_array_to_RS232  *********************************

	  Entity Name:               Entity Type:  Translation to English:

	  "char_array"               Input          An array of chars where you expect the 
	                                           reply from the RS232 conduit.

     "Desired_number_of_chars"   Input         The number maximum number of chars that
	                                           you want inserted into  "char_array"   
    
	  "Desired_number_of_chars"  Output        The actual number of chars that were actually 
	                                          deliverd to the RS232 conduit.  

      Function return value:      Translation:

	  !=0X00                     Not all of the RS232 adjustments were made
	   0X01                      PORT has been selected 
	   0X02                      BAUD has been selected
	   0X04                      Number of data bits has been selected.
	   0X08                      Parity has been selected
	   0X10                      Stop bits have been selected
	   0X20                      TIMEOUTS NOT selected
	   0X08                      Read fucntion reported an error


	  ******************  _____________________________  **********************************/

	    unsigned char MS_VS_2005_RS232_manager::Receive_char_array_from_RS232( char char_array[], int &Desired_number_of_chars )
		{

		 unsigned char Reply_Value;

		   unsigned char Entgang_buffer[0XFFFF];// never expect to transmitt more than 0XFFFF!!

		   int          Actual_number_of_bytes_recieved;

		   int         Read_Sucess_Flag;
		   Reply_Value=0X00;


		   if (Desired_number_of_chars<256)
		   {
			   Desired_number_of_chars=256;// Avoids plenty of confusion
		   }




		    Reply_Value =_pvt_User_selections_Examination();// DO NOT proceed if COM PORT does not work

			if (0X00!= Reply_Value)
			{
				return 0X3F-Reply_Value;
			}

			

			if (0X00==_pvt_RS232_conduit_has_been_generated)
			{

				Generate_RS232_conduit();//Implicit desire for a functioning conduit

			}


			Read_Sucess_Flag = ReadFile(_pvt_COM_port_file_pntr, Entgang_buffer, Desired_number_of_chars, (LPDWORD)((void *)&Actual_number_of_bytes_recieved), NULL);


			if (0==Read_Sucess_Flag)
			{
				printf("MS_VS_2005_RS232_manager::Receive_char_array_from_RS232 has reported a problem!\n");

				Reply_Value = 0X80;//
				
			}

			_pvt_unsigned_char_array_to_char_array(Entgang_buffer, char_array, (unsigned int) Actual_number_of_bytes_recieved);

			Desired_number_of_chars = (int) Actual_number_of_bytes_recieved;

			char_array[Desired_number_of_chars]='\0';//Always return a NULL terminated array.

			 return   Reply_Value;
		}


     unsigned char MS_VS_2005_RS232_manager::_pvt_char_array_to_unsigned_char_array(char Source_char_array[], unsigned char Destination_unsigned_char_array[], unsigned int Array_Length)
	{

		unsigned char reply_message=0;
		unsigned int   byte_index;

		 for (byte_index=0;byte_index<Array_Length;byte_index++)
		 {
          Destination_unsigned_char_array[byte_index] = (unsigned char) Source_char_array[byte_index];
		 }

        return reply_message;
	}


    unsigned char    MS_VS_2005_RS232_manager::_pvt_unsigned_char_array_to_char_array(unsigned char Source_unsigned_char_array[], char Destination_char_array[], unsigned int Array_Length)
	{
		unsigned char reply_message=0;
		unsigned int   byte_index;

		 for (byte_index=0;byte_index<Array_Length;byte_index++)
		 {
          Destination_char_array[byte_index] = (char) Source_unsigned_char_array[byte_index];
		 }

        return reply_message;
      };


	void   MS_VS_2005_RS232_manager::Obliterate_RS232_conduit()
	{

           if (0!=_pvt_RS232_conduit_has_been_generated)
		   {

			    CloseHandle(_pvt_COM_port_file_pntr);
		   }

	}




   void          MS_VS_2005_RS232_manager::Report_existing_COM_port_adjustments( )
   {

	   char Commnd_prompt_message[256];


	   if (0X00==(0X01&_pvt_PORT_adjustments_monitor))
	   {
		   printf("ERROR! COM PORT number must be specified prior to a ''settings'' QUIRY!\n");
		   return;
	   }

	   	//strncpy(_pvt_COM_NUM_ID_array,"COM",256);
		//strncat(_pvt_COM_NUM_ID_array,Number_array,256);

		  strncpy(Commnd_prompt_message,"mode ",256);
		  strncat(Commnd_prompt_message,_pvt_COM_NUM_ID_array,256);
		  strncat(Commnd_prompt_message," /STATUS ",256);


		  printf("\n Ignore the ''DEVICE CURRENTLY NOT AVALIBLE'' message! \n\n");
		 system (Commnd_prompt_message);

		 printf("\n \n \n");


		 if (0==_pvt_RS232_conduit_has_been_generated)
		 {
           printf("   ''MS_VS_2005_RS232_manager'' can not report ''TIMEOUT'' aka latency data until\n");
		   printf(" RS232 conduit has been sucessfully created.\n\n\n");
		   return;
		 }



		  printf("   ''ReadIntervalTimeout'' (mS)      = %i\n\n\n",_pvt_COM_port_latency_magic.ReadIntervalTimeout  );
		  //printf("   ''ReadTotalTimeoutMultiplier''  = %i\n",_pvt_COM_port_latency_magic.ReadTotalTimeoutMultiplier  );
		  //printf("   ''ReadIntervalTimeout''         = %i\n",_pvt_COM_port_latency_magic.ReadTotalTimeoutConstant  );
		  //printf("   ''WriteTotalTimeoutMultiplier'' = %i\n", _pvt_COM_port_latency_magic.WriteTotalTimeoutMultiplier  );
		  //printf("   ''WriteTotalTimeoutConstant''   = %i\n", _pvt_COM_port_latency_magic.WriteTotalTimeoutConstant  );


	


   }


    void   MS_VS_2005_RS232_manager::Hardware_latency_mS(DWORD TIME_out_miliseconds)
	{
		
		_pvt_Time_out_miliseconds = TIME_out_miliseconds;

		_pvt_PORT_adjustments_monitor = _pvt_PORT_adjustments_monitor |0X20;

	}


#endif





/****************** References and notes ********************************

 1) The atavistic progenitor code:
 
    visit: 
    http://www.teuniz.net/RS-232/ 

 2) "MODE" command 
 
    A) https://www.microsoft.com/resources/documentation/windows/xp/all/proddocs/en-us/mode.mspx?mfr=true
 
    B) Also try the following internet search terms: "mode command syntax serial port Microsoft"

	C) At a command prompt type "mode /?" the operating system will spew some gibberish at you.


 3)  "CreateFileA"
     
	 A) http://msdn.microsoft.com/en-us/library/windows/desktop/aa363858(v=vs.85).aspx

	 B) Also try the following internet search terms: "CreateFileA" only view Microsoft pages


 4)  "DWORD WINAPI GetLastError(void);"
     There literally thousands of these error codes!!

	 A) http://msdn.microsoft.com/en-us/library/windows/desktop/ms681381(v=vs.85).aspx

  5) "COMMITEMEOUTS"
     A) http://msdn.microsoft.com/en-us/library/aa450505.aspx
	 B)  B) Also try the following internet search terms: "COMMITEMEOUTS" only view Microsoft pages


****************** ____ **********************************/




	  
/*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*/
